{-# LANGUAGE FlexibleContexts #-}
{-# OPTIONS_GHC -Wall         #-}

module HarmTrace.Audio.Parser where

import HarmTrace.Audio.ChordTypes
import HarmTrace.Base.Parsing hiding (pComma,pQuotedString,pParentheticalString)

--------------------------------------------------------------------------------
-- data parsers
--------------------------------------------------------------------------------

-- Beat Parsing                                  
parseBeatData :: Parser BeatTrackerData
parseBeatData = pListSep_ng pLineEnd pLine <* pLineEnd where
  pLine = opt pLabel "" *> pNumData <* opt (pComma *> pQuotedString) ""

-- Chroma parsing                  
parseChordinoData :: Parser ChordinoData
parseChordinoData =  pListSep_ng pLineEnd pChordinoLine <* pLineEnd where
  pChordinoLine = const convert <$> opt pLabel ""
                                <*> pList1Sep (pSym ',') pNumData 
  convert :: [NumData] -> ChordinoLine -- shift the chorma to match C .. B
  convert l | length l == 25 = ChordinoLine h (shift 3 a) (shift 3 b)
            | otherwise = error ("Wrong list length of " ++ show (length l))
            where (h:t) = l
                  (a,b) = splitAt 12 t

-- rotates the elements in the list with n positions
shift :: Int -> [a] -> [a]
shift p l = b ++ a where (a,b) = splitAt p l                 

reverseShift :: Int -> [a] -> [a]
reverseShift p l = b ++ a where (a,b) = splitAt ((length l) - p) l                 

-- key parsing
parseKeyStrengthData :: Parser KeyStrengthData  
parseKeyStrengthData =  pListSep_ng pLineEnd pKeyLine <* pLineEnd where
  pKeyLine = convert <$> pList1Sep (pSym ',') pNumData  
  convert :: [NumData] -> ChordinoLine
  convert l | length l == 26 = ChordinoLine h a (tail b)
            | otherwise = error ("Wrong list length of " ++ show (length l))
            where (h:t) = l
                  (a,b) = splitAt 12 t  

-- key 12 dim vector chroms
parseChromaData :: Parser [ChordinoLine]
parseChromaData =  pListSep_ng pLineEnd pCrmLine <* pLineEnd where
  pCrmLine = convert <$> pList1Sep (pSym ',') pNumData  
  -- This is a bit of a hack, but I do not want to rewrite all the functions 
  -- again for a very similar data type that only has one 12-dim chroma vector
  convert :: [NumData] -> ChordinoLine   
  convert l | length t == 12 = ChordinoLine h (shift 3 t) [] -- hence we make this []
            | otherwise = error ("Wrong list length of " ++ show (length l))
            where (h:t) = l
                  
                  
--------------------------------------------------------------------------------
-- Basic parsers
--------------------------------------------------------------------------------
                  
pNumData :: Parser NumData
pNumData = pDoubleRaw

pComma :: Parser Char
pComma = pSym ','

pParentheticalString :: Char -> Parser String
pParentheticalString d = pSym d *> pList pNonQuoteVChar <* pSym d where
  pNonQuoteVChar = pSatisfy (\c -> visibleChar c && c /= d) 
                   (Insertion ("Character in a string set off from main text" ++
                    "by delimiter, e.g. double-quotes or comment token") 'y' 5)
  visibleChar c = '\032' <= c && c <= '\126'

pQuotedString :: Parser String
pQuotedString = pParentheticalString '"'

pLabel :: Parser String
pLabel = (pQuotedString `opt` "") <* pComma
